import cv2
import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif'] = ['simHei']
plt.rcParams['axes.unicode_minus'] = False

# 读取照片
image_zeropad = cv2.imread("file/zeropad.jpg", cv2.IMREAD_GRAYSCALE)

# 设置填充区域大小
top, bottom, left, right = 50, 50, 50, 50

# 不填充
image_no_padded = image_zeropad.copy()

# 零填充
image_zero_padded = cv2.copyMakeBorder(
    image_zeropad, top, bottom, left, right,
    borderType=cv2.BORDER_CONSTANT,
    value=[0, 0, 0]
)

# 原图填充
image_copy_padded = cv2.copyMakeBorder(
    image_zeropad, top, bottom, left, right,
    borderType=cv2.BORDER_REPLICATE
)

# 进行平滑滤波（均值滤波）
kernel_size = (5, 5)
image_no_padded_blur = cv2.blur(image_no_padded, kernel_size)
image_zero_padded_blur = cv2.blur(image_zero_padded, kernel_size)
image_copy_padded_blur = cv2.blur(image_copy_padded, kernel_size)

# 展示图像
plt.subplot(2,3,1)
plt.imshow(image_no_padded, cmap='gray')
plt.title('不填充')
plt.axis('off')

plt.subplot(2,3,2)
plt.imshow(image_zero_padded, cmap='gray')
plt.title('零填充')
plt.axis('off')

plt.subplot(2,3,3)
plt.imshow(image_copy_padded, cmap='gray')
plt.title('原图填充')
plt.axis('off')

plt.subplot(2,3,4)
plt.imshow(image_no_padded_blur, cmap='gray')
plt.title('不填充+均值滤波')
plt.axis('off')

plt.subplot(2,3,5)
plt.imshow(image_zero_padded_blur, cmap='gray')
plt.title('零填充+均值滤波')
plt.axis('off')

plt.subplot(2,3,6)
plt.imshow(image_copy_padded_blur, cmap='gray')
plt.title('原图填充+均值滤波')
plt.axis('off')

plt.tight_layout()
plt.show()